@@ -220,7 +220,11 @@ xfce_appfinder_model_init (XfceAppfinderModel *model)
   model->menu = garcon_menu_new_applications ();
   appfinder_refcount_debug_add (G_OBJECT (model->menu), "main menu");
 
+#if GLIB_CHECK_VERSION (2, 32, 0)
+  model->collect_thread = g_thread_new ("Collector", xfce_appfinder_model_collect_thread, model);
+#else
   model->collect_thread = g_thread_create (xfce_appfinder_model_collect_thread, model, TRUE, NULL);
+#endif
 }
 
 
@@ -1333,8 +1337,12 @@ xfce_appfinder_model_collect_items (GarconMenu           *menu,
       if (!garcon_menu_directory_get_visible (directory))
         return FALSE;
 
-      /* this way we only have two levels */
-      if (category == NULL)
+      /* this way we only have 1 level of categories, but
+       * skip the directory of the root menu because in some
+       * menu files (gnome-menus for example) this puts
+       * everything under "Applications" */
+      if (category == NULL
+          && garcon_menu_get_parent (menu) != NULL)
         category = directory;
     }
 
