@@ -119,6 +119,10 @@ xfce_appfinder_preferences_init (XfceAppfinderPreferences *preferences)
   xfconf_g_property_bind (preferences->channel, "/always-center", G_TYPE_BOOLEAN,
                           G_OBJECT (object), "active");
 
+  object = gtk_builder_get_object (GTK_BUILDER (preferences), "enable-service");
+  xfconf_g_property_bind (preferences->channel, "/enable-service", G_TYPE_BOOLEAN,
+                          G_OBJECT (object), "active");
+
   icons = gtk_builder_get_object (GTK_BUILDER (preferences), "icon-view");
   xfconf_g_property_bind (preferences->channel, "/icon-view", G_TYPE_BOOLEAN,
                           G_OBJECT (icons), "active");
@@ -181,11 +185,18 @@ xfce_appfinder_preferences_response (GtkWidget                *window,
   appfinder_return_if_fail (GTK_IS_DIALOG (window));
   appfinder_return_if_fail (XFCE_IS_APPFINDER_PREFERENCES (preferences));
 
-  g_signal_handler_disconnect (preferences->channel, preferences->property_watch_id);
+  if (response_id == GTK_RESPONSE_HELP)
+    {
+      xfce_dialog_show_help (GTK_WINDOW (window), "xfce4-appfinder", "preferences", NULL);
+    }
+  else
+    {
+      g_signal_handler_disconnect (preferences->channel, preferences->property_watch_id);
 
-  gtk_widget_destroy (window);
+      gtk_widget_destroy (window);
 
-  g_object_unref (G_OBJECT (preferences));
+      g_object_unref (G_OBJECT (preferences));
+    }
 }
 
 
@@ -381,6 +392,7 @@ xfce_appfinder_preferences_action_changed (XfconfChannel            *channel,
   gint           unique_id;
   GObject       *store;
   UpdateContext  context;
+  gint           offset = 0;
 
   if (prop_name == NULL)
     return;
@@ -389,8 +401,9 @@ xfce_appfinder_preferences_action_changed (XfconfChannel            *channel,
     {
       xfce_appfinder_preferences_action_populate (preferences);
     }
-  else if (sscanf (prop_name, "/actions/action-%d/pattern", &unique_id) == 1
-           && G_VALUE_HOLDS_STRING (value))
+  else if (G_VALUE_HOLDS_STRING (value)
+           && sscanf (prop_name, "/actions/action-%d%n", &unique_id, &offset) == 1
+           && g_strcmp0 (prop_name + offset, "/pattern") == 0)
     {
       context.unique_id = unique_id;
       context.value = value;
