@@ -127,6 +127,8 @@ struct _XfceAppfinderWindow
 
   XfceAppfinderActions       *actions;
 
+  GtkIconTheme               *icon_theme;
+
   GtkEntryCompletion         *completion;
 
   XfconfChannel              *channel;
@@ -202,7 +204,6 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   GtkWidget          *bbox;
   GtkWidget          *button;
   GtkEntryCompletion *completion;
-  GtkIconTheme       *icon_theme;
   gint                integer;
 
   window->channel = xfconf_channel_get ("xfce4-appfinder");
@@ -370,9 +371,10 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   image = gtk_image_new_from_stock (GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (button), image);
 
-  icon_theme = gtk_icon_theme_get_for_screen (gtk_window_get_screen (GTK_WINDOW (window)));
-  g_signal_connect_swapped (G_OBJECT (icon_theme), "changed",
+  window->icon_theme = gtk_icon_theme_get_for_screen (gtk_window_get_screen (GTK_WINDOW (window)));
+  g_signal_connect_swapped (G_OBJECT (window->icon_theme), "changed",
       G_CALLBACK (xfce_appfinder_window_icon_theme_changed), window);
+  g_object_ref (G_OBJECT (window->icon_theme));
 
   /* load categories in the model */
   xfce_appfinder_window_category_set_categories (NULL, window);
@@ -402,6 +404,11 @@ xfce_appfinder_window_finalize (GObject *object)
   g_signal_handler_disconnect (window->channel, window->property_watch_id);
   g_signal_handler_disconnect (window->model, window->categories_changed_id);
 
+  /* release our reference on the icon theme */
+  g_signal_handlers_disconnect_by_func (G_OBJECT (window->icon_theme),
+      xfce_appfinder_window_icon_theme_changed, window);
+  g_object_unref (G_OBJECT (window->icon_theme));
+
   g_object_unref (G_OBJECT (window->model));
   g_object_unref (G_OBJECT (window->category_model));
   g_object_unref (G_OBJECT (window->completion));
@@ -923,6 +930,13 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
                           XFCE_APPFINDER_MODEL_COLUMN_TITLE, &title,
                           XFCE_APPFINDER_MODEL_COLUMN_URI, &uri, -1);
 
+      /* custom command don't have an uri */
+      if (uri == NULL)
+        {
+          g_free (title);
+          return FALSE;
+        }
+
       uri_is_local = g_str_has_prefix (uri, "file://");
 
       menu = gtk_menu_new ();
@@ -1102,6 +1116,7 @@ xfce_appfinder_window_entry_activate (GtkEditable         *entry,
                                       XfceAppfinderWindow *window)
 {
   GtkTreePath *path;
+  gboolean     cursor_set = FALSE;
 
   if (gtk_widget_get_visible (window->paned))
     {
@@ -1111,6 +1126,8 @@ xfce_appfinder_window_entry_activate (GtkEditable         *entry,
             {
               gtk_tree_view_set_cursor (GTK_TREE_VIEW (window->view), path, NULL, FALSE);
               gtk_tree_path_free (path);
+
+              cursor_set = TRUE;
             }
         }
       else if (gtk_icon_view_get_visible_range (GTK_ICON_VIEW (window->view), &path, NULL))
@@ -1118,9 +1135,14 @@ xfce_appfinder_window_entry_activate (GtkEditable         *entry,
           gtk_icon_view_select_path (GTK_ICON_VIEW (window->view), path);
           gtk_icon_view_set_cursor (GTK_ICON_VIEW (window->view), path, NULL, FALSE);
           gtk_tree_path_free (path);
+
+          cursor_set = TRUE;
         }
 
-      gtk_widget_grab_focus (window->view);
+      if (cursor_set)
+        gtk_widget_grab_focus (window->view);
+      else
+        xfce_appfinder_window_execute (window);
     }
   else if (gtk_widget_get_sensitive (window->button_launch))
     {
@@ -1433,13 +1455,18 @@ xfce_appfinder_window_row_activated (XfceAppfinderWindow *window)
 static void
 xfce_appfinder_window_icon_theme_changed (XfceAppfinderWindow *window)
 {
+  appfinder_return_if_fail (XFCE_IS_APPFINDER_WINDOW (window));
+
   if (window->icon_find != NULL)
     g_object_unref (G_OBJECT (window->icon_find));
   window->icon_find = xfce_appfinder_model_load_pixbuf (GTK_STOCK_FIND, XFCE_APPFINDER_ICON_SIZE_48);
 
   /* drop cached pixbufs */
-  xfce_appfinder_model_icon_theme_changed (window->model);
-  xfce_appfinder_category_model_icon_theme_changed (window->category_model);
+  if (G_LIKELY (window->model != NULL))
+    xfce_appfinder_model_icon_theme_changed (window->model);
+
+  if (G_LIKELY (window->category_model != NULL))
+    xfce_appfinder_category_model_icon_theme_changed (window->category_model);
 
   /* update state */
   xfce_appfinder_window_entry_changed (window);
@@ -1452,6 +1479,7 @@ static gboolean
 xfce_appfinder_window_execute_command (const gchar          *text,
                                        GdkScreen            *screen,
                                        XfceAppfinderWindow  *window,
+                                       gboolean              only_custom_cmd,
                                        gboolean             *save_cmd,
                                        GError              **error)
 {
@@ -1474,6 +1502,8 @@ xfce_appfinder_window_execute_command (const gchar          *text,
     return FALSE;
   else if (action_cmd != NULL)
     text = action_cmd;
+  else if (only_custom_cmd)
+    return FALSE;
 
   if (IS_STRING (text))
     {
@@ -1505,13 +1535,17 @@ xfce_appfinder_window_execute (XfceAppfinderWindow *window)
   gchar        *cmd = NULL;
   gboolean      regular_command = FALSE;
   gboolean      save_cmd;
-
-  if (!gtk_widget_get_sensitive (window->button_launch))
-    return;
+  gboolean      only_custom_cmd = FALSE;
 
   screen = gtk_window_get_screen (GTK_WINDOW (window));
   if (gtk_widget_get_visible (window->paned))
     {
+      if (!gtk_widget_get_sensitive (window->button_launch))
+        {
+          only_custom_cmd = TRUE;
+          goto entry_execute;
+        }
+
       if (xfce_appfinder_window_view_get_selected (window, &model, &iter))
         {
           gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &orig, &iter);
@@ -1520,17 +1554,22 @@ xfce_appfinder_window_execute (XfceAppfinderWindow *window)
           if (!result && regular_command)
             {
               gtk_tree_model_get (model, &iter, XFCE_APPFINDER_MODEL_COLUMN_COMMAND, &cmd, -1);
-              result = xfce_appfinder_window_execute_command (cmd, screen, window, NULL, &error);
+              result = xfce_appfinder_window_execute_command (cmd, screen, window, FALSE, NULL, &error);
               g_free (cmd);
             }
         }
     }
   else
     {
+      if (!gtk_widget_get_sensitive (window->button_launch))
+        return;
+
+      entry_execute:
+
       text = gtk_entry_get_text (GTK_ENTRY (window->entry));
       save_cmd = TRUE;
 
-      if (xfce_appfinder_window_execute_command (text, screen, window, &save_cmd, &error))
+      if (xfce_appfinder_window_execute_command (text, screen, window, only_custom_cmd, &save_cmd, &error))
         {
           if (save_cmd)
             result = xfce_appfinder_model_save_command (window->model, text, &error);
@@ -1539,14 +1578,17 @@ xfce_appfinder_window_execute (XfceAppfinderWindow *window)
         }
     }
 
-  gtk_entry_set_icon_from_stock (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_PRIMARY,
-                                 result ? NULL : GTK_STOCK_DIALOG_ERROR);
-  gtk_entry_set_icon_tooltip_text (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_PRIMARY,
-                                   error != NULL ? error->message : NULL);
+  if (!only_custom_cmd)
+    {
+      gtk_entry_set_icon_from_stock (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_PRIMARY,
+                                     result ? NULL : GTK_STOCK_DIALOG_ERROR);
+      gtk_entry_set_icon_tooltip_text (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_PRIMARY,
+                                       error != NULL ? error->message : NULL);
+    }
 
   if (error != NULL)
     {
-      g_warning ("Failed to execute: %s", error->message);
+      g_printerr ("%s: failed to execute: %s\n", G_LOG_DOMAIN, error->message);
       g_error_free (error);
     }
 
